/*
   PS2 Intro - ps2dev Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to display a black screen
   using the Official Playstation 2 Linux kit and the official
   ps2dev library.
*/


// Demo: PS2 Intro - ps2dev.


// Inlcudes...
#include<ps2gs.h>
#include<stdio.h>


// Playstation 2 Globals
int gs_FileDesc;
ps2_gs_gparam *gs_gParam;
ps2_gs_dbuff gs_dBuffer;
ps2_gs_finish gs_Finish;

int frame = 0;

// Function Prototypes
int InitializePS2();
int Acquire();
void RenderScene();
void Shutdown();


int main()
{
   // Gets a pointer to the graphics synthesiser.
   gs_gParam = ps2_gs_get_gparam();

   // Initialize the PS2 stuff.
   if(InitializePS2() != PS2_GS_VC_ACQ_SUCCESS)
      {
         printf("Initialize fail!\n\n");
         Shutdown();
         return 0;
      }

   printf("Initialize success!\n");
   printf("Preparing to draw the scene.\n");

   // Loop through and render the scene a set number of times.
   // If you want to exit the program before this loop is done
   // just hit CTRL-C.
   for(frame = 0; frame < 30000; frame++)
   {
      RenderScene();
   }

   printf("Shutting down...\n");

   // Release all resources.
   Shutdown();
   printf("Exiting the Playstation 2 program.\n\n");

   return 0;
}


int InitializePS2()
{
   // Acquire the system.
   if(Acquire() == PS2_GS_VC_ACQ_FAILURE)
      return PS2_GS_VC_ACQ_FAILURE;

   // Set the double buffer mode.  Takes a object to set, a pixel
   // storage mode which can be PS2_GS_PSMCT32, PS2_GS_PSMCT24,
   // PS2_GS_PSMCT16, PS2_GS_PSM16S, a width, a height, flag for how
   // the z buffer test should work which can be PS2_GS_TEST_ZTST_NEVER,
   // PS2_GS_TEST_ZTST_ALWAYS, PS2_GS_ZGREATER, or PS2_GS_ZEQUAL, a
   // storage for how the z buffer should be kept which can be
   // PS2_GS_PSMZ32, PS2_GS_PSMZ24, PS2_GS_PSMZ16, and PS2_GS_PSMZ16S,
   // and a flag to clear the screen.  1 means true, 0 false.
   ps2_gs_set_dbuff(&gs_dBuffer, PS2_GS_PSMCT32, gs_gParam->width,
                    gs_gParam->height, PS2_GS_ZGREATER, PS2_GS_PSMZ24, 1);

   // Start the display up.
   ps2_gs_start_display(0);

   return PS2_GS_VC_ACQ_SUCCESS;
}


int Acquire()
{
   // Open the ps2 graphics sythesiser.  A value less than 0 passed to
   // this means you want it to open on its own, greater or equal to zero
   // means you are sending a handle to a device.
   gs_FileDesc = ps2_gs_open(-1);

   // Error checking.
   if(gs_FileDesc < 0) return PS2_GS_VC_ACQ_FAILURE;

   // Reset the graphics sythesiser.  First is mode.  0 means reset
   // everything, 1 means stop drawing and destroy the current primitive.
   // Next is inter which can set the gs to no interlace or interlace. 
   // Third is omode.  This can be PS2_GS_VESA if you are viewing on a PC monitor,
   // PS2_GS_DTV if your on a digital tv, PS2_GS_NTSC if your on a japanese or
   // US tv, or PS2_GS_PAL if your own a european tv.  Fourth is ffmode which
   // if you have PS2_GS_FIELD scans every other line while PS2_GS_FRAME scans
   // every line.  Only used if in interlace mode.  5th is the resolution.
   // This can be PS2_GS_640x480, PS2_GS_800x600, PS2_GS_1024x768, or PS2_GS_1280x1024
   // for VESA mode or PS2_GS_480P, PS2_GS_720P, or PS2_GS_1080I for any of the TV modes.
   // Last you have the refresh rate which can be PS2_GS_60Hz or PS2_GS_75Hz.
   ps2_gs_reset(0, PS2_GS_NOINTERLACE, PS2_GS_NTSC, PS2_GS_FIELD,
                PS2_GS_720P, PS2_GS_60Hz);

   return PS2_GS_VC_ACQ_SUCCESS;
}


void RenderScene()
{
   int r = 0, g = 255, b = 0;

   // Wait for v sync.  Then swap the buffers since the last one
   // is ready to be displayed.
   ps2_gs_sync_v(0);
   ps2_gs_swap_dbuff(&gs_dBuffer, frame);

   // Set the background color for both buffers.  The macro takes
   // a r, g, b, a color values as well as q (which is used for
   // texture mapping).
   *(__u64*)&gs_dBuffer.clear0.rgbaq = PS2_GS_SETREG_RGBAQ(r, g, b, 0, 0);
   *(__u64*)&gs_dBuffer.clear1.rgbaq = PS2_GS_SETREG_RGBAQ(r, g, b, 0, 0);

   // We can use frame & 1 to determine which buffer should be displayed
   // to the screen.  Either the 1st buffer or the second.
   ps2_gs_put_drawenv((frame & 1) ? &gs_dBuffer.giftag1 :
                                    &gs_dBuffer.giftag0);

   // The first tells the GS to wait until all pixels are drawn.
   // The second actually forces the wait.  When it does the program
   // continues.
   ps2_gs_set_finish(&gs_Finish);
   ps2_gs_wait_finish(&gs_Finish);
}


void Shutdown()
{
   // As long as we had a value gs_FileDesc we close down.
   if(gs_FileDesc >= 0)
      {
         ps2_gs_close();
         gs_FileDesc = -1;
      }
}


// Recap:

// As you can see every something as simple as clearing the
// screen takes a bit of work and knowledge.  Lucky for us
// the manuals that come with the kit are good.  PS2 is not
// known for being easy to develop for and now you have a small
// taste of what the companies must go through.


// Copyright February 2005
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com